/***************************************************************************
 *   Copyright (C) 2007 by                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "acs.h"
#include <unistd.h>
// #include <qtango.h>
#include <QMessageBox>
#include <qtimer.h>


	char tmBuffer[80];
	int noSrvRetry;
/*
  timestamp() 
  send a timestamp to standard output
*/
void timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}

acs::acs(QWidget *parent) : QWidget(parent)
{
	noSrvRetry = 0;
	QSettings settings("fermi", "acs");
	// load (or set) path for images
	if (settings.contains("imgpath")) {
		imgpath = settings.value("imgpath").toString();
		// cout << imgpath.toStdString() << endl;
	}
	else {
		settings.setValue("imgpath", "/runtime/fermi/share/images/acs");
		// settings.setValue("imgpath", "/home/zambonl/devel/qt/fermi/acs/img");
	}
	if (!imgpath.endsWith("/")) {
		imgpath = imgpath + "/";
	}
	// get host name
	char hostname[256];
	strcpy(hostname, getenv("DISPLAY"));
	if (!strncmp(hostname, ":0", 2)) {
		if (gethostname(hostname, 255) < 0) {
			cout << "invalid hostname" << endl;
			strcpy(hostname, "nohost:0");
		}
	}
	cout << "hostname: " << hostname << endl;
	// load (or set) control location
	if (settings.contains("controlLocation")) {
		controlLocation = settings.value("controlLocation").toString();
	}
	else {
		settings.setValue("controlLocation", "Sala Controllo Anello");
		controlLocation = "controlLocation";
	}
	// settings.setValue("master", "ca-booster:0.0");
	// settings.setValue("masterA0", "kaioh:0.0,kaioh-clone:0.0");

	// load (or set) master
	if (settings.contains("master")) {
		master = settings.value("master").toString().contains(hostname);
	}
	else {
		settings.setValue("master", "ca-booster:0.0");
		master = false;
	}
	std::string a = settings.value("master").toString().toStdString();
	cout << a << ", master: " << (master? "true": "false") << endl;
	// load (or set) master for A0
	if (settings.contains("masterA0")) {
		masterA0 = settings.value("masterA0").toString().contains(hostname);
	}
	else {
		settings.setValue("masterA0", "kaioh:0.0,kaioh-clone:0.0");
		masterA0 = false;
	}
	// std::string b = settings.value("masterA0").toString().toStdString();
	// cout << b << ", masterA0: " << (masterA0? "true": "false") << endl;
	// masterA0 = true;
	// setup User Interface
	try {
		ui.setupUi(this);
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot setup UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	
	
	// <==== TODO check master
	master = masterA0 = true;
	// ui.controlFrame->setHidden(true);
	
	
	cout << "connecting" << endl;
	// connect to control access server
	try {
		plc_server = new Tango::DeviceProxy("f/access_control/safety");
		// plc_server = new Tango::DeviceProxy("ken/plc/fsafety");
		// plc_server = new Tango::DeviceProxy("booster/accessi/plc_accessi");
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	cout << "connected" << endl; // exit(0);
	refreshCount = 0;
	refresh();
	// init timer
	try {
		timer = new QTimer(this);
		timer->setInterval(500);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI" << endl;
	}
	catch (...) {int
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// init button action
	try {
		// open list of staff inside
		connect(ui.A0PresenceButton, SIGNAL(clicked()), this, SLOT(open_inA0_list()));
		inA0_list_process = new QProcess(this);
		connect(ui.staffListButton, SIGNAL(clicked()), this, SLOT(open_staff_list()));
		staff_list_process = new QProcess(this);
		// switch control
		connect(ui.controlSwitchButton, SIGNAL(clicked()), this, SLOT(switch_control_location()));
		permission_process = new QProcess(this);
		// open permission panel
		connect(ui.enableInfoButton_1, SIGNAL(clicked()), this, SLOT(open_stat_linac_shutdown()));
		stat_shutdown_process = new QProcess(this);
		// open plant panel
		connect(ui.enableInfoButton_2, SIGNAL(clicked()), this, SLOT(open_stat_linac_off()));
		stat_off_process = new QProcess(this);
		// open status panels
		connect(ui.enableInfoButton_3, SIGNAL(clicked()), this, SLOT(open_stat_linac_on()));
		stat_on_process = new QProcess(this);
		connect(ui.enableInfoButton_9, SIGNAL(clicked()), this, SLOT(open_rfinhibit_panel()));
		rfinhibit_process = new QProcess(this);
		connect(ui.enableInfoButton_7, SIGNAL(clicked()), this, SLOT(open_rfpcgun_panel()));
		rfpcgun_process = new QProcess(this);
		connect(ui.enableInfoButton_6, SIGNAL(clicked()), this, SLOT(open_klystat()));
		klystat_process = new QProcess(this);
		connect(ui.SwitchPickupButton, SIGNAL(clicked()), this, SLOT(open_klyswitch()));
		connect(ui.enableInfoButton_8, SIGNAL(clicked()), this, SLOT(open_klyswitch()));
		klyswitch_process = new QProcess(this);
		// open doors detailed panels
		connect(ui.linacDoorButton, SIGNAL(clicked()), this, SLOT(open_linac_doors()));
		linac_doors_process = new QProcess(this);
		connect(ui.e0DoorButton, SIGNAL(clicked()), this, SLOT(open_e0_doors()));
		e0_doors_process = new QProcess(this);
		connect(ui.e1DoorButton, SIGNAL(clicked()), this, SLOT(open_e1_doors()));
		e1_doors_process = new QProcess(this);
		connect(ui.e2DoorButton, SIGNAL(clicked()), this, SLOT(open_e2_doors()));
		e2_doors_process = new QProcess(this);
		connect(ui.e3DoorButton, SIGNAL(clicked()), this, SLOT(open_e3_doors()));
		e3_doors_process = new QProcess(this);
		connect(ui.tlDoorButton, SIGNAL(clicked()), this, SLOT(open_tl_doors()));
		tl_doors_process = new QProcess(this);
		// open emergency buttons detailed panel
		connect(ui.emergencyButton, SIGNAL(clicked()), this, SLOT(open_emergency_buttons()));
		emergency_buttons_process = new QProcess(this);
		// open BST detailed panels
		connect(ui.bst_linacButton, SIGNAL(clicked()), this, SLOT(open_bst_linac_panel()));
		bst_linac_process = new QProcess(this);
		// open search detailed panel
		connect(ui.watchInfoButton_0, SIGNAL(clicked()), this, SLOT(open_search_panel()));
		connect(ui.watchInfoButton_1, SIGNAL(clicked()), this, SLOT(open_search_panel()));
		connect(ui.watchInfoButton_2, SIGNAL(clicked()), this, SLOT(open_search_panel()));
		connect(ui.watchInfoButton_3, SIGNAL(clicked()), this, SLOT(open_search_panel()));
		connect(ui.watchInfoButton_4, SIGNAL(clicked()), this, SLOT(open_search_panel()));
		connect(ui.watchInfoButton_5, SIGNAL(clicked()), this, SLOT(open_search_panel()));
		search_process = new QProcess(this);
		// open BST detailed panel
		connect(ui.bstInfoButton, SIGNAL(clicked()), this, SLOT(open_bstinfo_panel()));
		bstinfo_process = new QProcess(this);
		// connect(ui.scrapersInfoButton_1, SIGNAL(clicked()), this, SLOT(open_scrapersinfo_panel()));
		// scrapersinfo_process = new QProcess(this);
		// switch booster map
		// connect(ui.mapButton, SIGNAL(clicked()), this, SLOT(open_map()));
		map = 0;
		connect(ui.openMapButton, SIGNAL(clicked()), this, SLOT(open_map_panel()));
		map_process = new QProcess(this);
		// open TOP-UP Error panel
		// connect(ui.topupError_Button, SIGNAL(clicked()), this, SLOT(open_topup_error()));
		// topup_error_process = new QProcess(this);
		// connect(ui.tunnelWatch_label, SIGNAL(clicked()), this, SLOT(open_bst()));
		bst_process = new QProcess(this);
		// open log page on Firefox browser
		connect(ui.alarmListButton, SIGNAL(clicked()), this, SLOT(open_alarmlist_panel()));
		alarmlist_process = new QProcess(this);
		// open hardware failure monitoring panel
		connect(ui.hardwareButton, SIGNAL(clicked()), this, SLOT(open_hardware_panel()));
		hardware_process = new QProcess(this);
		// open version monitoring panel
		connect(ui.versionButton, SIGNAL(clicked()), this, SLOT(open_version_panel()));
		// operations reserved to master only
		// if (master) {
		  // open access doors
		  connect(ui.A0OpenButton, SIGNAL(clicked()), this, SLOT(open_A0door()));
		  // connect(ui.A1OpenButton, SIGNAL(clicked()), this, SLOT(open_A1door()));
		  connect(ui.A0OpenButton, SIGNAL(pressed()), this, SLOT(open2_A0door()));
		  connect(ui.A0OpenButton, SIGNAL(released()), this, SLOT(close_A0door()));
		  // open/close beam stoppers
		  connect(ui.bst_linac_OpenCmd_Button, SIGNAL(clicked()), this, SLOT(open_bst_linac()));
		  connect(ui.bst_linac_CloseCmd_Button, SIGNAL(clicked()), this, SLOT(close_bst_linac()));
		  /*
		  connect(ui.bst_bts11_OpenCmd_Button, SIGNAL(clicked()), this, SLOT(open_bst_bts11()));
		  connect(ui.bst_bts11_CloseCmd_Button, SIGNAL(clicked()), this, SLOT(close_bst_bts11()));
		  connect(ui.bst_bts12_OpenCmd_Button, SIGNAL(clicked()), this, SLOT(open_bst_bts12()));
		  connect(ui.bst_bts12_CloseCmd_Button, SIGNAL(clicked()), this, SLOT(close_bst_bts12()));
		  connect(ui.bst_bts13_OpenCmd_Button, SIGNAL(clicked()), this, SLOT(open_bst_bts13()));
		  connect(ui.bst_bts13_CloseCmd_Button, SIGNAL(clicked()), this, SLOT(close_bst_bts13()));
		  */
		  // connect(ui.bst_bl_CloseCmd_Button, SIGNAL(clicked()), this, SLOT(close_bst_bl()));
		  // acknowledge emergency
		  connect(ui.emergencyRestoreButton, SIGNAL(clicked()), this, SLOT(emergencyRestore()));
		  // restore
		  connect(ui.A0RestoreButton, SIGNAL(clicked()), this, SLOT(a0Restore()));
		  // connect(ui.A1RestoreButton, SIGNAL(clicked()), this, SLOT(a1Restore()));
		  // request watch
		  connect(ui.A0WatchButton, SIGNAL(clicked()), this, SLOT(watchRequest()));
		  // connect(ui.A1WatchButton, SIGNAL(clicked()), this, SLOT(watchRequest()));
		// }
		// controlSwitchButton
		// connect(ui.controlSwitchButton, SIGNAL(clicked()), this, SLOT(switch_master()));
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI buttons" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	/*
		ui.rb_lt01->setIcon(QPixmap(imgpath+""));
		ui.rb_lt02->setIcon(QPixmap(imgpath+""));
		ui.rb_lt03->setIcon(QPixmap(imgpath+""));
		// ui.rb_lt04->setIcon(QPixmap(imgpath+""));
		// ui.rb_lt05->setIcon(QPixmap(imgpath+""));
		// ui.rb_lt06->setIcon(QPixmap(imgpath+""));
		ui.rb_lt07->setIcon(QPixmap(imgpath+""));
		ui.rb_e0->setIcon(QPixmap(imgpath+""));
		// ui.rb_e1->setIcon(QPixmap(imgpath+""));
		// ui.rb_e2->setIcon(QPixmap(imgpath+""));
		// ui.rb_e3->setIcon(QPixmap(imgpath+""));
		// ui.rb_tl->setIcon(QPixmap(imgpath+""));
	*/
	// init map
	// ui.mapButton->setIcon(QPixmap(imgpath+"/floor_small.png"));
	// init info buttons
	ui.watchInfoButton_0->setIcon(QPixmap(imgpath+"/info.png"));
	ui.watchInfoButton_1->setIcon(QPixmap(imgpath+"/info.png"));
	ui.watchInfoButton_2->setIcon(QPixmap(imgpath+"/info.png"));
	ui.watchInfoButton_3->setIcon(QPixmap(imgpath+"/info.png"));
	ui.watchInfoButton_4->setIcon(QPixmap(imgpath+"/info.png"));
	ui.watchInfoButton_5->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_1->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_2->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_3->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_6->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_7->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_8->setIcon(QPixmap(imgpath+"/info.png"));
	ui.enableInfoButton_9->setIcon(QPixmap(imgpath+"/info.png"));
	ui.bstInfoButton->setIcon(QPixmap(imgpath+"/info.png"));
	/*
	ui.topupError_Button->setIcon(QPixmap(imgpath+"/info.png"));
	ui.scrapersInfoButton_1->setIcon(QPixmap(imgpath+"/info.png"));
	*/
	// init bpm close buttons 
	ui.bst_linac_CloseCmd_Button->setIcon(QPixmap(imgpath+"/close_cmd.png"));
	ui.A0PresenceButton->setIcon(QPixmap(imgpath+"/list.png"));
	ui.A0OpenButton->setIcon(QPixmap(imgpath+"/open.png"));
	/*
	ui.bst_bts11_CloseCmd_Button->setIcon(QPixmap(imgpath+"/close_cmd.png"));
	ui.bst_bts12_CloseCmd_Button->setIcon(QPixmap(imgpath+"/close_cmd.png"));
	ui.bst_bts13_CloseCmd_Button->setIcon(QPixmap(imgpath+"/close_cmd.png"));
	mySetTextColor(ui.bst_bts21_Stat_label, "", "#000000");
	*/
//	ui.bst_bl_CloseCmd_Button->setIcon(QPixmap(imgpath+"/close_cmd_red.png"));
	// init restore buttons 
	ui.A0RestoreButton->setIcon(QPixmap(imgpath+"/restore.png"));
	// ui.A1RestoreButton->setIcon(QPixmap(imgpath+"/restore.png"));
	ui.A0WatchButton->setIcon(QPixmap(imgpath+"/watch_request.png"));
	ui.emergencyRestoreButton->setIcon(QPixmap(imgpath+"/restore_small.png"));
	ui.emergencyRestoreButton_tl->setIcon(QPixmap(imgpath+"/restore_small.png"));
	// init open panel buttons
	ui.alarmListButton->setIcon(QPixmap(imgpath+"/alarm_list.png"));
	ui.controlSwitchButton->setIcon(QPixmap(imgpath+"/control_switch.png"));
	ui.openMapButton->setIcon(QPixmap(imgpath+"/map.png"));
	ui.staffListButton->setIcon(QPixmap(imgpath+"/staff_list.png"));
	ui.versionButton->setIcon(QPixmap(imgpath+"/version.png"));
	ui.hardwareButton->setIcon(QPixmap(imgpath+"/hardware.png"));
	ui.SwitchPickupButton->setIcon(QPixmap(imgpath+"/switchpickup.png"));
	// init siren
	ui.A0_siren_label->setPixmap(QPixmap(imgpath+"/siren.png"));
	ui.A0_permission_label->setPixmap(QPixmap(imgpath+"/permissions.png"));
	ui.emergencyButton->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.emergencyButton_tl->setIcon(QPixmap(imgpath+"/alarm_off.png"));
	// ui.linac_label->setPixmap(QPixmap(imgpath+"Linac_1250.png"));
	ui.consentA0_0->setPixmap(QPixmap(imgpath+"/permission_off.png"));
	ui.consentA0_1->setPixmap(QPixmap(imgpath+"/permission_off.png"));
	ui.consentA0_2->setPixmap(QPixmap(imgpath+"/permission_off.png"));
	// init doors
	// ui.e0_2->setIcon(QPixmap(imgpath+"/e0_2_open.png"));
	// ui.i1_e0->setIcon(QPixmap(imgpath+"/i1_e0_open.png"));
	// cout << imgpath.toStdString() << "/Linac_plant_1250.png" << endl;
	// ui.linac_label_door_0->setPixmap(QPixmap(imgpath+"/linac_door1_red.png"));
	// set style
	style = " -style=windows";
	plcstat = false;
	ui.noPlc->hide();
	/*
	ui.eb_lt01->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt02->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt03->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt04->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt05->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt06->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt07->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt08->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt19->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt20->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt21->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt22->setIcon(QPixmap(imgpath+"/alarm_on.png"));
	ui.eb_lt01->hide();
	ui.eb_lt02->hide();
	ui.eb_lt03->hide();
	ui.eb_lt04->hide();
	ui.eb_lt05->hide();
	ui.eb_lt06->hide();
	ui.eb_lt07->hide();
	ui.eb_lt08->hide();
	ui.eb_lt19->hide();
	ui.eb_lt20->hide();
	ui.eb_lt21->hide();
	ui.eb_lt22->hide();
	*/
}

acs::~acs()
{
}

int acs::myWriteCommand(const char *cmdName)
{
	try {
		plc_server->command_inout(cmdName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error writing command: " << cmdName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception writing command: " << cmdName << endl;
	}
	return 0;
}
    
int acs::myReadShortint(const char *attrName, short int *attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> *attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}
    
int acs::myReadString(const char *attrName, string &strstat)
{
	try {
		plc_server->read_attribute(attrName) >> strstat;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}
    
int acs::myReadShortArray(const char *attrName, vector<short> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acs::myReadBool(const char *attrName, bool *attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> *attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}
    
int acs::myReadBoolArray(const char *attrName, vector<bool> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acs::myReadAttribute(const char *attrName)
{
	try {
		myAttribute = plc_server->read_attribute(attrName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acs::mySetColor(QFrame *widget, const char *color, QPalette::ColorRole role=QPalette::Dark)
{
	QPalette button;
	button.setColor(QPalette::Light, QColor(color));
	button.setColor(role, QColor(color));
	widget->setPalette(button);
	return 0;
}

int acs::mySetTextColor(QLabel *widget, const char *text, const char *color)
{
	QPalette button;
	widget->setText(text);
	button.setColor(QPalette::WindowText, QColor(color));
	widget->setPalette(button);
	return 0;
}

int acs::mySetTextColor(QLabel *widget, const char *text, const char *color, const char *bgcolor)
{
	QPalette button;
	widget->setText(text);
	button.setColor(QPalette::Window, QColor(bgcolor));
	button.setColor(QPalette::WindowText, QColor(color));
	widget->setPalette(button);
	return 0;
}






void acs::open_A0door()
{
	cout << "open_A0door " << endl;
	// operations reserved to master only
	if (!master and !masterA0) {
		QMessageBox::information(this, "Apertura Negata", "Apertura porta consentita solo da workstation controllo accessi\nin sala controllo anello accanto alle telecamere", QMessageBox::Ok);
	}
	else if (controlLocation == "Sala Controllo Anello") {
		if (master) {
			myWriteCommand("LinacDoorOpen");
		}
		else {
			QMessageBox::information(this, "Apertura Negata", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	else {
		if (masterA0) {
			myWriteCommand("LinacDoorOpen");
		}
		else {
			QMessageBox::information(this, "Apertura Negata", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
}

void acs::open2_A0door()
{
	cout << "open_A0door(), pressed " << endl;

}

void acs::close_A0door()
{
	cout << "open_A0door(), released " << endl;

}

void acs::open_A1door()
{
	// operations reserved to master only
	if (master) {
		if (controlLocation == "Sala Controllo Anello") {
			myWriteCommand("open_door_bo");
		}
		else {
			QMessageBox::information(this, "Apertura Negata", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	else {
		QMessageBox::information(this, "Apertura Negata", "Apertura porta A1 consentita solo da workstation controllo accessi\nin sala controllo anello accanto alle telecamere", QMessageBox::Ok);
	}	
	cout << "open_A1door " << endl;

}

void acs::a0Restore()
{
	// operations reserved to master only
	if (!master and !masterA0) {
		QMessageBox::information(this, "Ripristino Negato", "Ripristino consentito solo da workstation controllo accessi\nin sala controllo anello accanto alle telecamere", QMessageBox::Ok);
	}
	else if (controlLocation == "Sala Controllo Anello") {
		if (master) {
			myWriteCommand("LinacDoorReset");
		}
		else {
			QMessageBox::information(this, "Ripristino Negato", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	else {
		if (masterA0) {
			myWriteCommand("LinacDoorReset");
		}
		else {
			QMessageBox::information(this, "Ripristino Negato", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	// myWriteCommand("Ripr_in_inj");
	cout << "a0Restore" << endl;
}

void acs::a1Restore()
{
	// operations reserved to master only
	if (master) {
		if (controlLocation == "Sala Controllo Anello") {
			myWriteCommand("Ripr_in_boost");
		}
		else {
			QMessageBox::information(this, "Ripristino Negato", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	else {
		QMessageBox::information(this, "Ripristino Negato", "Ripristino consentito solo da workstation controllo accessi\nin sala controllo anello accanto alle telecamere", QMessageBox::Ok);
	}
	// myWriteCommand("Ripr_in_boost");
	cout << "a1Restore" << endl;
}

void acs::watchRequest()
{
	// operations reserved to master only
	if (!master and !masterA0) {
		QMessageBox::information(this, "Richiesta ronda negata", "Richiesta ronda consentita solo da workstation controllo accessi\nin sala controllo anello accanto alle telecamere", QMessageBox::Ok);
	}
	else if (controlLocation == "Sala Controllo Anello") {
		if (master) {
			// watch request procedure
			try {
				int r = QMessageBox::question(this, "Conferma richiesta ronda", "Conferma richiesta ronda", "Richiesta ronda", "Annulla");
				if (r == 0) {
					myWriteCommand("LinacSearchRequest");
				}
			}
			catch (const Tango::DevFailed &e) {
				timestamp();
				cout << tmBuffer << "Non posso richiedere la ronda" << endl;
			}
		}
		else {
			QMessageBox::information(this, "Richiesta ronda negata", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	else {
		if (masterA0) {
			// watch request procedure
			try {
				int r = QMessageBox::question(this, "Conferma richiesta ronda", "Conferma richiesta ronda", "Richiesta ronda", "Annulla");
				if (r == 0) {
					myWriteCommand("LinacSearchRequest");
				}
			}
			catch (const Tango::DevFailed &e) {
				timestamp();
				cout << tmBuffer << "Non posso richiedere la ronda" << endl;
			}
		}
		else {
			QMessageBox::information(this, "Richiesta ronda negata", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	cout << "watchRequest" << endl;
}

void acs::emergencyRestore()
{
	// operations reserved to master only
	if (!master and !masterA0) {
		QMessageBox::information(this, "Ripristino Negato", "Ripristino consentito solo da workstation controllo accessi\nin sala controllo anello accanto alle telecamere", QMessageBox::Ok);
	}
	else if (controlLocation == "Sala Controllo Anello") {
		if (master) {
			myWriteCommand("LinacEmeAck");
		}
		else {
			QMessageBox::information(this, "Ripristino Negato", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	else {
		if (masterA0) {
			myWriteCommand("LinacEmeAck");
		}
		else {
			QMessageBox::information(this, "Ripristino Negato", "Controlla bottone: CAMBIA CONTROLLO PSA / SALA CONTROLLO", QMessageBox::Ok);
		}
	}
	// myWriteCommand("Ack_eme");
	cout << "Ack_eme" << endl;
}

void acs::open_inA0_list()
{
	if (inA0_list_process->state() == QProcess::Running) {
		inA0_list_process->kill();
		inA0_list_process->waitForFinished(3000);			
	}	
	inA0_list_process->start("acslistinalinac"+style);

}

void acs::open_inA1_list()
{
	if (inA1_list_process->state() == QProcess::Running) {
		inA1_list_process->kill();
		inA1_list_process->waitForFinished(3000);			
	}	
	inA1_list_process->start("acslistina1"+style);

}

void acs::open_staff_list()
{
	if (staff_list_process->state() == QProcess::Running) {
		staff_list_process->kill();
		staff_list_process->waitForFinished(3000);			
	}	
	staff_list_process->start("acsliststaff"+style);

}

void acs::open_linac_doors()
{
	if (linac_doors_process->state() == QProcess::Running) {
		linac_doors_process->kill();
		linac_doors_process->waitForFinished(3000);			
	}	
	linac_doors_process->start("acsdoorl"+style);
}

void acs::open_e0_doors()
{
	if (e0_doors_process->state() == QProcess::Running) {
		e0_doors_process->kill();
		e0_doors_process->waitForFinished(3000);			
	}	
	e0_doors_process->start("acsdoore0"+style);
}

void acs::open_e1_doors()
{
	if (e1_doors_process->state() == QProcess::Running) {
		e1_doors_process->kill();
		e1_doors_process->waitForFinished(3000);			
	}	
	e1_doors_process->start("acsdoore1"+style);
}

void acs::open_e2_doors()
{
	if (e2_doors_process->state() == QProcess::Running) {
		e2_doors_process->kill();
		e2_doors_process->waitForFinished(3000);			
	}	
	e2_doors_process->start("acsdoore2"+style);
}

void acs::open_e3_doors()
{
	if (e3_doors_process->state() == QProcess::Running) {
		e3_doors_process->kill();
		e3_doors_process->waitForFinished(3000);			
	}	
	e3_doors_process->start("acsdoore3"+style);
}

void acs::open_tl_doors()
{
	if (tl_doors_process->state() == QProcess::Running) {
		tl_doors_process->kill();
		tl_doors_process->waitForFinished(3000);			
	}	
	tl_doors_process->start("acsdoortl"+style);
}

void acs::open_emergency_buttons()
{
	// cout << "open_emergency_buttons() " << endl;
	emergency_buttons_process->start("acsemergency"+style);
}

void acs::open_bst_linac_panel()
{
	if (bst_linac_process->state() != QProcess::Running) {
		bst_linac_process->start("acsbst Bst_linac Part_bstB_13_1 BST_B14.1"+style);
	}	
}

void acs::open_bstinfo_panel()
{
	if (bstinfo_process->state() != QProcess::Running) {
		bstinfo_process->kill();
		bstinfo_process->waitForFinished(3000);			
	}
	bstinfo_process->start("acsbst bst1.1 Part_bstl.01 BST_LINAC"+style);	
}

void acs::open_bst11info_panel()
{
	if (bst11info_process->state() != QProcess::Running) {
		bst11info_process->kill();
		bst11info_process->waitForFinished(3000);			
	}
	bst11info_process->start("acsbst11info"+style);	
}

void acs::open_bst12info_panel()
{
	if (bst12info_process->state() != QProcess::Running) {
		bst12info_process->kill();
		bst12info_process->waitForFinished(3000);			
	}
	bst12info_process->start("acsbst12info"+style);	
}

void acs::open_bst13info_panel()
{
	if (bst13info_process->state() != QProcess::Running) {
		bst13info_process->kill();
		bst13info_process->waitForFinished(3000);			
	}
	bst13info_process->start("acsbst13info"+style);	
}

void acs::open_bst14info_panel()
{
	QMessageBox::information(this, "BST_BTS2.1", "Per l'abilitazione all'apertura verificare sul PC di interlock macchina:\n - lo stato delle valvole da vuoto dell'anello\n - lo stato degli ID\n - il raffreddamento dell'anello (flussostati, pressostati, termoswitch, etc)", QMessageBox::Ok);
}

void acs::open_stat_linac_shutdown()
{
	// cout << "open_stat_linac_shutdown()" << endl; return;
	if (stat_shutdown_process->state() == QProcess::Running) {
		stat_shutdown_process->kill();
		stat_shutdown_process->waitForFinished(3000);			
	}	
	stat_shutdown_process->start("acslinacshutdown"+style);
}

void acs::open_stat_linac_off()
{
	if (stat_off_process->state() == QProcess::Running) {
		stat_off_process->kill();
		stat_off_process->waitForFinished(3000);			
	}	
	stat_off_process->start("acslinacoff"+style);
}

void acs::open_stat_linac_on()
{
	if (stat_on_process->state() == QProcess::Running) {
		stat_on_process->kill();
		stat_on_process->waitForFinished(3000);			
	}	
	stat_on_process->start("acslinacon"+style);
}

void acs::open_klyswitch()
{
	if (klyswitch_process->state() == QProcess::Running) {
		klyswitch_process->kill();
		klyswitch_process->waitForFinished(3000);			
	}	
	klyswitch_process->start("acsklyswitch"+style);
}

void acs::open_klystat()
{
	if (klystat_process->state() == QProcess::Running) {
		klystat_process->kill();
		klystat_process->waitForFinished(3000);			
	}	
	klystat_process->start("acsklystat"+style);
}

void acs::open_rfinhibit_panel()
{
	if (rfinhibit_process->state() == QProcess::Running) {
		rfinhibit_process->kill();
		rfinhibit_process->waitForFinished(3000);			
	}
	rfinhibit_process->start("acsinhibitrf"+style);
}

void acs::open_rfpcgun_panel()
{
	if (rfpcgun_process->state() == QProcess::Running) {
		rfpcgun_process->kill();
		rfpcgun_process->waitForFinished(3000);			
	}
	short int mod;
	myReadShortint("Linac_mod_on_pcgun", &mod);	
	cout << (mod==0? "acsinhibitrfks": "acsinhibitrfk1") << endl;
	rfpcgun_process->start((mod==0? "acsinhibitrfks": "acsinhibitrfk1")+style);
}

void acs::open_permission_panel()
{
	if (permission_process->state() == QProcess::Running) {
		permission_process->kill();
		permission_process->waitForFinished(3000);			
	}	
	permission_process->start("acspermission"+style);
}

void acs::open_plants_panel()
{
	if (plants_process->state() == QProcess::Running) {
		plants_process->kill();
		plants_process->waitForFinished(3000);			
	}	
	plants_process->start("acsplants"+style);
}

void acs::open_topup_error()
{
	if (topup_error_process->state() == QProcess::Running) {
		topup_error_process->kill();
		topup_error_process->waitForFinished(3000);			
	}
	topup_error_process->start("acstopuperror"+style);	
}

void acs::open_search_panel()
{
	if (search_process->state() == QProcess::Running) {
		search_process->kill();
		search_process->waitForFinished(3000);	
	}
	search_process->start("acssearch"+style);
}

void acs::open_map()
{
	// ui.mapButton->setIcon(QPixmap(imgpath+(map? "/floor_small.png": "/roof_small.png")));
	map = 1 - map;
}

void acs::open_map_panel()
{
	if (map_process->state() == QProcess::Running) {
		map_process->kill();
		map_process->waitForFinished(3000);
	}
	map_process->start("acsmap"+style);	
}

void acs::open_hardware_panel()
{
	if (hardware_process->state() == QProcess::Running) {
		hardware_process->kill();
		hardware_process->waitForFinished(3000);
	}
	hardware_process->start("acshardware"+style);
}

void acs::refresh()
{
	vector<bool> stat, stat2, stat3, stat4;
	bool boolstat;
	string strstat;
	// stat.size(16);
	refreshCount = 1 - refreshCount;
	Tango::DeviceAttribute StateFlags;
	if (plc_server == NULL) {
		cout << "Cannot connect server" << endl;
		return;
	}
	// set PLC status
	if (myReadBool("Plc_status", &boolstat)) {
		noSrvRetry = 0;
		if (boolstat) {
			ui.noPlc->hide();
			mySetTextColor(ui.A0_plc_label, "PLC ON", "white", "black");
			mySetColor(ui.A0_plc_label, "#ffffff", QPalette::Window);
		}
		else {
			mySetTextColor(ui.A0_plc_label, refreshCount? "NO PLC": "", "black", refreshCount? "#ffff00": "white");
			mySetColor(ui.A0_plc_label, "#ffd800", QPalette::Window);
			ui.noPlc->show();
		}
		/*
		// cout << "boolstat" << endl;
		if (!plcstat && boolstat) {
			QMessageBox::information(this, "PLC NOK", "\n\n\n\n        ATTENZIONE!                        \n\n\n        PLC non raggiungibile                                 \n\n\n\n\n\n\n\n\n\n", QMessageBox::Ok);
		}
		// cout << "mySetTextColor" << endl;
		mySetTextColor(ui.A0_plc_label, boolstat? "PLC OK": "PLC NOK", boolstat? "black": "#e00000", "white");
		// mySetTextColor(ui.A1_plc_label, boolstat? "PLC OK": "PLC NOK", boolstat? "black": "#e00000", "white");
		plcstat = boolstat;
		// ui.A0_plc_label->setText(boolstat? "PLC ON": "PLC OFF");
		*/
	}
	else {
		noSrvRetry++;
		mySetColor(ui.A0_plc_label, "#ffd800", QPalette::Window);
		mySetTextColor(ui.A0_plc_label, refreshCount? "NO SRV": "", "black", refreshCount? "#ffff00": "white");
		// mySetTextColor(ui.A1_plc_label, refreshCount? "NO SRV": "", "black", refreshCount? "#ffff00": "white");
		// ui.A0_plc_label->setText(refreshCount? "PLC ???": "");
		timestamp();
		cout << tmBuffer << "error: cannot connect to PLC" << endl;
		if (noSrvRetry > 3 && noSrvRetry%40) return;
	}
	// set linac external badge board
	if (myReadBoolArray("Linac_key_ext", stat) and myReadBoolArray("Linac_key_release", stat2)) {
		ui.keyA0ext_00->setPixmap(QPixmap(imgpath+(stat[0]? (stat2[0]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_01->setPixmap(QPixmap(imgpath+(stat[1]? (stat2[1]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_02->setPixmap(QPixmap(imgpath+(stat[2]? (stat2[2]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_03->setPixmap(QPixmap(imgpath+(stat[3]? (stat2[3]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_04->setPixmap(QPixmap(imgpath+(stat[4]? (stat2[4]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_05->setPixmap(QPixmap(imgpath+(stat[5]? (stat2[5]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_06->setPixmap(QPixmap(imgpath+(stat[6]? (stat2[6]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_07->setPixmap(QPixmap(imgpath+(stat[7]? (stat2[7]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_08->setPixmap(QPixmap(imgpath+(stat[8]? (stat2[8]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_09->setPixmap(QPixmap(imgpath+(stat[9]? (stat2[9]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_10->setPixmap(QPixmap(imgpath+(stat[10]? (stat2[10]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_11->setPixmap(QPixmap(imgpath+(stat[11]? (stat2[11]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_12->setPixmap(QPixmap(imgpath+(stat[12]? (stat2[12]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_13->setPixmap(QPixmap(imgpath+(stat[13]? (stat2[13]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_14->setPixmap(QPixmap(imgpath+(stat[14]? (stat2[14]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
		ui.keyA0ext_15->setPixmap(QPixmap(imgpath+(stat[15]? (stat2[15]? "internal_key_on.png": "external_key_on.png"): "external_key_off.png")));
	}
	else {
		ui.keyA0ext_00->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_01->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_02->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_03->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_04->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_05->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_06->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_07->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_08->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_09->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_10->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_11->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_12->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_13->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_14->setPixmap(QPixmap(imgpath+("external_key_off.png")));
		ui.keyA0ext_15->setPixmap(QPixmap(imgpath+("external_key_off.png")));
	}
	// set linac internal badge board
	if (myReadBoolArray("Linac_key_int", stat)) {
		ui.keyA0int_00->setPixmap(QPixmap(imgpath+(stat[0]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_01->setPixmap(QPixmap(imgpath+(stat[1]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_02->setPixmap(QPixmap(imgpath+(stat[2]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_03->setPixmap(QPixmap(imgpath+(stat[3]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_04->setPixmap(QPixmap(imgpath+(stat[4]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_05->setPixmap(QPixmap(imgpath+(stat[5]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_06->setPixmap(QPixmap(imgpath+(stat[6]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_07->setPixmap(QPixmap(imgpath+(stat[7]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_08->setPixmap(QPixmap(imgpath+(stat[8]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_09->setPixmap(QPixmap(imgpath+(stat[9]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_10->setPixmap(QPixmap(imgpath+(stat[10]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_11->setPixmap(QPixmap(imgpath+(stat[11]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_12->setPixmap(QPixmap(imgpath+(stat[12]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_13->setPixmap(QPixmap(imgpath+(stat[13]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_14->setPixmap(QPixmap(imgpath+(stat[14]? "internal_key_on.png": "internal_key_off.png")));
		ui.keyA0int_15->setPixmap(QPixmap(imgpath+(stat[15]? "internal_key_on.png": "internal_key_off.png")));
	}
	else {
		ui.keyA0int_00->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_01->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_02->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_03->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_04->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_05->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_06->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_07->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_08->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_09->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_10->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_11->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_12->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_13->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_14->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
		ui.keyA0int_15->setPixmap(QPixmap(imgpath+("internal_key_off.png")));
	}
	// set linac presence
	if (myReadBoolArray("Linac_glob_signals", stat)) {
		ui.A0PresenceButton->setIcon(QPixmap(imgpath+((refreshCount && (!stat[1]))? "listred.png": "list.png")));
		// ui.pr_lock->setIcon(QPixmap(imgpath+(stat[9]? "lock.png": "unlock.png")));
	}
	else {
		ui.A0PresenceButton->setIcon(QPixmap(imgpath+"list.png"));
		// ui.pr_lock->setIcon(QPixmap(imgpath+"unlock.png"));
	}
	bool free_access = false;
	// set access permission
	if (myReadBoolArray("Linac_access_state", stat)) {
		QPalette button;
		const QColor color;
		free_access = stat[10];
		if (stat[10]) mySetTextColor(ui.A0_stat_label, "LIBERO", "#00aa00", "white");
		if (stat[12]) mySetTextColor(ui.A0_stat_label, "CONTROLLATO", "#e0e000", "black");
		if (stat[11]) mySetTextColor(ui.A0_stat_label, "VIETATO", "#e00000", "white");
		ui.A0WatchButton->setIcon(QPixmap(imgpath+(stat[12]? "/watch_request.png": "/watch_request_inhibit.png")));
	}
	else {
		mySetTextColor(ui.A0_stat_label, "???", "#e0e000", "black");
	}
	// set hourglass siren and radiation warning
	if (myReadBoolArray("Linac_sir_buz", stat)) {
		ui.A0_hourglass_label->setPixmap(QPixmap(imgpath+(stat[1]? "hourglass.png": "")));
		ui.A0_radiation_label->setPixmap(QPixmap(imgpath+(stat[2] || stat[3]? "radiation.png": "")));
		ui.A0_siren_label->setPixmap(QPixmap(imgpath+(!stat[10]? "siren.png": "siren_on.png")));
	}
	else {
		ui.A0_hourglass_label->setPixmap(QPixmap(imgpath+""));
		ui.A0_radiation_label->setPixmap(QPixmap(imgpath+""));
		ui.A0_siren_label->setPixmap(QPixmap(imgpath+"siren.png"));
	}
	// set linac input process
	if (myReadBoolArray("Linac_glob_signals", stat)) {
		ui.consentA0_2->setPixmap(QPixmap(imgpath+(!stat[7]? "permission_on.png": "permission_off.png")));
	}
	else {
		ui.consentA0_2->setPixmap(QPixmap(imgpath+"permission_off.png"));
	}
	// set linac badge acknowledge
	if (myReadString("Linac_badge_ack", strstat)) {
		ui.userNameA0Label->setText(strstat.c_str());
	}
	else {
		ui.userNameA0Label->setText("???");
	}
	/*
	// is restore necessary
	if (myReadAttribute("Linac_access_signals")) {
		myAttribute >> stat;
	}
	*/
	bool e0, e1, e2, e3, etl, el;
	bool eh0, eh1, eh2, eh3, ehtl, ehl;
	bool en0, en1, en2, en3, entl, enl;
	en0=en1=en2=en3=entl=enl=true;
	e0=e1=e2=e3=etl=el=true;
	eh0=eh1=eh2=eh3=ehtl=ehl=true;
	// set escape doors e0
	if (myReadBoolArray("Linac_doors_e0", stat) && myReadBoolArray("Linac_doors_memoopen_e0", stat2)) {
		/*
		ui.e0_1_label->setIcon(QPixmap(imgpath+(!stat[0]? "/e0_1_label_red.png": (stat2[0]? "/e0_1_label_yellow.png": "/e0_1_label_white.png"))));
		ui.e0_1_door->setIcon(QPixmap(imgpath+(!stat[0]? "/e0_1_door_open.png": "/e0_1_door_close.png")));
		ui.e0_2->setIcon(QPixmap(imgpath+(!stat[1]? "/e0_2_red.png": (stat2[1]? "/e0_2_yellow.png": "/e0_2_white.png"))));
		ui.i1_e0->setIcon(QPixmap(imgpath+(!stat[2]? "/i1_e0_red.png": (stat2[2]? "/i1_e0_yellow.png": "/i1_e0_white.png"))));
		ui.i2_e0->setIcon(QPixmap(imgpath+(!stat[3]? "/i2_e0_red.png": (stat2[3]? "/i2_e0_yellow.png": "/i2_e0_white.png"))));
		*/
		e0 = !stat[0] || !stat[1] || !stat[2] || !stat[3];
		eh0 = stat2[0] || stat2[1] || stat2[2] || stat2[3];
		ui.e0DoorButton->setIcon(QPixmap(imgpath+(e0? "door_small_open.png": (eh0? "door_small_hold.png": "door_small_close.png"))));
		el = !stat[0];
		ehl = stat2[0];
	}
	else {
		ui.e0DoorButton->setIcon(QPixmap(imgpath+"door_small_unknown.png"));
		en0 = false;
	}
	// set escape doors e1
	if (myReadBoolArray("Linac_doors_e1", stat) && myReadBoolArray("Linac_doors_memoopen_e1", stat2)) {
		e1 = !stat[0] || !stat[1] || !stat[2] || !stat[3];
		eh1 = stat2[0] || stat2[1] || stat2[2] || stat2[3];
		ui.e1DoorButton->setIcon(QPixmap(imgpath+(e1? "door_small_open.png": (eh1? "door_small_hold.png": "door_small_close.png"))));
		el = etl || !stat[0];
		ehl = ehtl || stat2[0];
	}
	else {
		ui.e1DoorButton->setIcon(QPixmap(imgpath+"door_small_unknown.png"));
		en1 = false;
	}
	// set escape doors e2
	if (myReadBoolArray("Linac_doors_e2", stat) && myReadBoolArray("Linac_doors_memoopen_e2", stat2)) {
		e2 = !stat[0] || !stat[1] || !stat[2] || !stat[3];
		eh2 = stat2[0] || stat2[1] || stat2[2] || stat2[3];
		ui.e2DoorButton->setIcon(QPixmap(imgpath+(e2? "door_small_open.png": (eh2? "door_small_hold.png": "door_small_close.png"))));
		el = etl || !stat[0];
		ehl = ehtl || stat2[0];
	}
	else {
		ui.e2DoorButton->setIcon(QPixmap(imgpath+"door_small_unknown.png"));
		en2 = false;
	}
	// set escape doors
	if (myReadBoolArray("Linac_doors_e3", stat) && myReadBoolArray("Linac_doors_memoopen_e3", stat2)) {
		e3 = !stat[0] || !stat[1] || !stat[2] || !stat[3] || !stat[4];
		eh3 = stat2[0] || stat2[1] || stat2[2] || stat2[3] || stat2[4];
		ui.e3DoorButton->setIcon(QPixmap(imgpath+(e3? "door_small_open.png": (eh3? "door_small_hold.png": "door_small_close.png"))));
		el = etl || !stat[0] || !stat[3] || !stat[4];
		ehl = ehtl || stat2[0] || stat2[3] || stat2[4];
	}
	else {
		ui.e3DoorButton->setIcon(QPixmap(imgpath+"door_small_unknown.png"));
		en3 = false;
	}
	// set escape doors
	if (myReadBoolArray("Linac_doors_tl", stat) && myReadBoolArray("Linac_doors_memoopen_tl", stat2)) {
		etl = !stat[0] || !stat[1];
		ehtl = stat2[0] || stat2[1];
		ui.tlDoorButton->setIcon(QPixmap(imgpath+(etl? "door_small_open.png": (ehtl? "door_small_hold.png": "door_small_close.png"))));
		el = etl || !stat[0] || !stat[1];
		ehl = ehtl || stat2[0] || stat2[1];
	}
	else {
		ui.tlDoorButton->setIcon(QPixmap(imgpath+"door_small_unknown.png"));
		entl = false;
	}
	// set escape doors l
	if (myReadBoolArray("Linac_doors_l", stat) && myReadBoolArray("Linac_doors_memoopen_l", stat2)) {
		ui.doorLinac_qLabel->setPixmap(QPixmap(imgpath+(!stat[0]? "door_big_open.png": "door_big_close.png")));
		// ui.alinac->setIcon(QPixmap(imgpath+(!stat[0]? "alinac_red.png": (stat2[0]? "alinac_yellow.png": "alinac_white.png"))));
		// ui.pr->setIcon(QPixmap(imgpath+(!stat[1]? "pr_red.png": (stat2[1]? "pr_yellow.png": "pr_white.png"))));
		el = etl || !stat[0] || !stat[1];
		ehl = ehtl || stat2[0] || stat2[1];
	}
	else {
		ui.doorLinac_qLabel->setPixmap(QPixmap(imgpath+"door_big_unknown.png"));
		enl = false;
	}
	// set escape doors global
	if (myReadBoolArray("Linac_doors_ext", stat) && myReadBoolArray("Linac_doors_memoopen_ext", stat2) && en0 && en1 && en2 && en3 && entl && enl) {
		bool eg = !stat[0] || !stat[1] || !stat[2] || el;
		bool ehg = stat2[0] || stat2[1] || stat2[2] || ehl;
		ui.linacDoorButton->setIcon(QPixmap(imgpath+(eg? "door_small_open.png": (ehg? "door_small_hold.png": "door_small_close.png"))));
		// ui.i3_e0->setIcon(QPixmap(imgpath+(!stat[2]? "i3_e0_red.png": (stat2[2]? "i3_e0_yellow.png": "i3_e0_white.png"))));
	}
	else {
		ui.linacDoorButton->setIcon(QPixmap(imgpath+"door_small_unknown.png"));
	}
	// set beam stoppers
	if (myReadBoolArray("Linac_bst_1", stat)) {
		ui.bst_linacButton->setIcon(QPixmap(imgpath+(stat[0]? "bst_open.png": (stat[1]? "bst_close.png": (stat[2] || stat[3]? "bst_alarm.png": "bst_unknown.png")))));
		mySetTextColor(ui.bst_linac_Stat_label, (stat[4]? "   abilitato": "disabilitato"), (stat[4]? "#00aa00": "#e00000"));
		ui.bst_linac_OpenCmd_Button->setIcon(QPixmap(imgpath+(stat[7]? "open_cmd.png": "open_cmd_hinibit.png")));
		ui.bst_linac_OpenStat_qLabel->setPixmap(QPixmap(imgpath+(stat[0]? "open_on.png": "open_off.png")));
		ui.bst_linac_CloseStat_qLabel->setPixmap(QPixmap(imgpath+(stat[1]? "close_on.png": "close_off.png")));
	}
	else {
		ui.bst_linacButton->setIcon(QPixmap(imgpath+("bst_unknown.png")));
		mySetTextColor(ui.bst_linac_Stat_label, "ignoto", "#aaaaaa");
		ui.bst_linac_OpenCmd_Button->setIcon(QPixmap(imgpath+"open_cmd_hinibit.png"));
	}
	// search status
	if (myReadBoolArray("Linac_search_signals_l", stat)) {
		// todo check watch_ini stat[2]?
		mySetTextColor(ui.tunnelWatch_label, (stat[0]? "RONDA OK": (refreshCount? (stat[6]? "RONDA IN CORSO": "RONDA PENDENTE"): "")), (stat[0]? "#00a000": (stat[6]? "#e08000": "#e00000")));
	}
	else {
		mySetTextColor(ui.tunnelWatch_label, "???", "#000000");
	}
	if (myReadBoolArray("Linac_search_signals_e0", stat)) {
		mySetTextColor(ui.e0Watch_label, (stat[0]? "RONDA OK": (refreshCount? (stat[4]? "RONDA IN CORSO": "RONDA PENDENTE"): "")), (stat[0]? "#00a000": (stat[4]? "#e08000": "#e00000")));
	}
	else {
		mySetTextColor(ui.e0Watch_label, "???", "#000000");
	}	
	if (myReadBoolArray("Linac_search_signals_e1", stat)) {
		mySetTextColor(ui.e1Watch_label, (stat[0]? "RONDA OK": (refreshCount? (stat[5]? "RONDA IN CORSO": "RONDA PENDENTE"): "")), (stat[0]? "#00a000": (stat[5]? "#e08000": "#e00000")));
	}
	else {
		mySetTextColor(ui.e1Watch_label, "???", "#000000");
	}	
	if (myReadBoolArray("Linac_search_signals_e2", stat)) {
		mySetTextColor(ui.e2Watch_label, (stat[0]? "RONDA OK": (refreshCount? (stat[5]? "RONDA IN CORSO": "RONDA PENDENTE"): "")), (stat[0]? "#00a000": (stat[5]? "#e08000": "#e00000")));
	}
	else {
		mySetTextColor(ui.e2Watch_label, "???", "#000000");
	}	
	if (myReadBoolArray("Linac_search_signals_e3", stat)) {
		mySetTextColor(ui.e3Watch_label, (stat[0]? "RONDA OK": (refreshCount? (stat[5]? "RONDA IN CORSO": "RONDA PENDENTE"): "")), (stat[0]? "#00a000": (stat[5]? "#e08000": "#e00000")));
	}
	else {
		mySetTextColor(ui.e3Watch_label, "???", "#000000");
	}	
	if (myReadBoolArray("Linac_search_signals_tl", stat)) {
		mySetTextColor(ui.tlWatch_label, (stat[0]? "RONDA OK": (refreshCount? (stat[2]? "RONDA IN CORSO": "RONDA PENDENTE"): "")), (stat[0]? "#00a000": (stat[2]? "#e08000": "#e00000")));
	}
	else {
		mySetTextColor(ui.tlWatch_label, "???", "#000000");
	}	
	bool eme = false, old_eme = false;
	/* emergency button summary
	if (myReadBoolArray("Linac_emergency_buttons", stat)) {
		if (!stat[14] && refreshCount) ui.eb_lt02->show(); else ui.eb_lt02->hide();
		if (!stat[14] && refreshCount) ui.eb_lt03->show(); else ui.eb_lt03->hide();
		if (!stat[0] && refreshCount) ui.eb_lt04->show(); else ui.eb_lt04->hide();
		if (!stat[1] && refreshCount) ui.eb_lt05->show(); else ui.eb_lt05->hide();
		if (!stat[1] && refreshCount) ui.eb_lt06->show(); else ui.eb_lt06->hide();
		if (!stat[12] && refreshCount) ui.eb_lt07->show(); else ui.eb_lt07->hide();
		if (!stat[12] && refreshCount) ui.eb_lt08->show(); else ui.eb_lt08->hide();
		if (!stat[13] && refreshCount) ui.eb_lt19->show(); else ui.eb_lt19->hide();
		if (!stat[13] && refreshCount) ui.eb_lt20->show(); else ui.eb_lt20->hide();
		if (!stat[2] && refreshCount) ui.eb_lt21->show(); else ui.eb_lt21->hide();
		if (!stat[2] && refreshCount) ui.eb_lt22->show(); else ui.eb_lt22->hide();
		if (!stat[2] && refreshCount) ui.eb_lt01->show(); else ui.eb_lt01->hide();
		eme = !stat[14] || !stat[0] || !stat[1] || !stat[12] || !stat[13] || !stat[2];
		/*
		if (!eme) {
			if (myReadBoolArray("Linac_emergency_memo_buttons", stat)) {
				old_eme = !stat[14] || !stat[0] || !stat[1] || !stat[12] || !stat[13] || !stat[2];
			}
		}
		*
		if (myReadBoolArray("Linac_safety", stat2)) {
			ui.emergencyRestoreButton->setIcon(QPixmap(imgpath+(stat2[0]? "restore_small.png": "restore_small_hinibit.png")));
			ui.emergencyButton->setIcon(QPixmap(imgpath+(eme && refreshCount? "alarm_on.png": (!stat2[0]? "alarm_off.png": "alarm_hold.png"))));
		}
		else {
			ui.emergencyRestoreButton->setIcon(QPixmap(imgpath+"restore_small_hinibit.png"));
			ui.emergencyButton->setIcon(QPixmap(imgpath+(refreshCount? "alarm_on.png": "alarm_off.png")));
		}
		// if (refreshCount) ui.eb_lt02->show(); else ui.eb_lt02->hide();
	}
	else {
		ui.eb_lt02->hide();
		ui.eb_lt03->hide();
		ui.eb_lt04->hide();
		ui.eb_lt05->hide();
		ui.eb_lt06->hide();
		ui.eb_lt07->hide();
		ui.eb_lt08->hide();
		ui.eb_lt19->hide();
		ui.eb_lt20->hide();
		ui.eb_lt21->hide();
		ui.eb_lt22->hide();
		ui.eb_lt01->hide();
	}
	*/
	ui.frame_2->stackUnder(ui.title_bst);
	// hardware button summary
	if (myReadBoolArray("Linac_boards_disabled", stat)) {
		bool enab = false;
		for (unsigned int i=0; i<stat.size(); i++) {
			enab = enab || stat[i];
		}
		ui.hardwareButton->setIcon(QPixmap(imgpath+(enab && refreshCount? "hardware_alarm.png": "hardware.png")));
	}
	// set linac input process
	if (myReadBoolArray("Linac_access_signals", stat)) {
		ui.consentA0_0->setPixmap(QPixmap(imgpath+(stat[8] || free_access? "permission_on.png": "permission_off.png")));
		ui.consentA0_1->setPixmap(QPixmap(imgpath+(stat[9] || stat[10] || stat[11] || free_access? "permission_on.png": "permission_off.png")));
		ui.A0OpenButton->setIcon(QPixmap(imgpath+(stat[9]? "open.png": "open_hinibit.png")));
		ui.A0RestoreButton->setIcon(QPixmap(imgpath+(stat[35]? "restore.png": "restore_hinibit.png")));
		// check if A_Linac in operation is running
		if (stat[8]) {
			mySetTextColor(ui.A0_access_label, refreshCount? "INGRESSO IN CORSO": "", "#e0e000", "#0000e0");
			if (stat[28]) ui.messageA0Label->setText("Premi ripristino (timeout estrazione testimone in ingresso)");
			if (stat[29]) ui.messageA0Label->setText("Premi ripristino (timeout sblocco porta in ingresso)");
			if (stat[30]) ui.messageA0Label->setText("Premi ripristino (timeout apertura porta in ingresso)");
			if (stat[31]) ui.messageA0Label->setText("Premi ripristino (timeout chiusura porta in ingresso)");
			if (stat[32]) ui.messageA0Label->setText("Premi ripristino (timeout deposito testimone in ingresso)");
		}
		// check if A_Linac out operation is running
		else if (stat[16]) {
			mySetTextColor(ui.A0_access_label, refreshCount? "USCITA IN CORSO": "", "#e0e000", "#0000e0");
			if (stat[28]) ui.messageA0Label->setText("Premi ripristino (timeout deposito testimone in uscita)");
			if (stat[29]) ui.messageA0Label->setText("Premi ripristino (timeout sblocco porta in uscita)");
			if (stat[30]) ui.messageA0Label->setText("Premi ripristino (timeout apertura porta in uscita)");
			if (stat[31]) ui.messageA0Label->setText("Premi ripristino (timeout chiusura porta in uscita)");
			if (stat[32]) ui.messageA0Label->setText("Premi ripristino (timeout lettura badge in uscita)");
		}
		else {
			mySetTextColor(ui.A0_access_label, "", "#e0e000", "white");
			ui.messageA0Label->setText("");
		}
	}
	else {
		ui.consentA0_0->setPixmap(QPixmap(imgpath+"permission_off.png"));
		ui.consentA0_1->setPixmap(QPixmap(imgpath+"permission_off.png"));
		ui.A0OpenButton->setIcon(QPixmap(imgpath+"open.png"));
		mySetTextColor(ui.A0_access_label, "", "#e0e000", "white");
		ui.messageA0Label->setText("");
	}
	if (myReadBoolArray("Linac_state", stat)) {
		mySetColor(ui.linacShutdownLabel, stat[0]? "black": "#dddddd", QPalette::WindowText);
		mySetColor(ui.linacOffLabel, stat[1]? "black": "#dddddd", QPalette::WindowText);
		mySetColor(ui.linacOnLabel, stat[2]? "black": "#dddddd", QPalette::WindowText);
	}
	else {
		mySetColor(ui.linacShutdownLabel, "#dddddd", QPalette::WindowText);
		mySetColor(ui.linacOffLabel, "#dddddd", QPalette::WindowText);
		mySetColor(ui.linacOnLabel, "#dddddd", QPalette::WindowText);
	}
		if (myReadBoolArray("Linac_mod_positions", stat)) {
			mySetTextColor(ui.linacRfPcgunEnabLabel, stat[9]? "Abilitata": "Disabilitata", stat[9]? "black": (refreshCount? "black": "white"));
		}
		else {
			mySetTextColor(ui.linacRfPcgunEnabLabel, "???", "black");
		}
	if (myReadBoolArray("Linac_mod_abi_rf", stat)) { 
		// short int pcgun, s0;
		mySetTextColor(ui.linacRfTunnelEnabLabel, stat[3]? "Abilitata": "Disabilitata", stat[3]? "black": (refreshCount? "black": "white"));
	}
	else {
		mySetTextColor(ui.linacRfTunnelEnabLabel, "???", "black");
	}
	vector<short> switchArray;
	if (myReadShortArray("Linac_switches_config", switchArray) && myReadBoolArray("Linac_pickup", stat2)) {
		// cout << switchArray[3] << endl;
		bool rfPcGunOn = switchArray[3]==3 || switchArray[3]==4 || switchArray[3]==6 || switchArray[3]==8 || !stat2[0];
		bool rfPcGunOff = switchArray[3]==1 || switchArray[3]==2 || switchArray[3]==5 || switchArray[3]==7;
		mySetTextColor(ui.linacRfPcgunStatLabel, (rfPcGunOn? "ON": (rfPcGunOff? "OFF": "???")), (rfPcGunOn? "#e00000": (rfPcGunOff? "#00b000": "black")));
		bool rfS0aOn = switchArray[5]==2 || switchArray[5]==4 || switchArray[5]==7 || switchArray[5]==8;
		bool rfS0aOff = switchArray[5]==1 || switchArray[5]==3 || switchArray[5]==5 || switchArray[5]==6;
		bool rfS0bOn = switchArray[4]==2 || switchArray[4]==4 || switchArray[4]==7 || switchArray[4]==8;
		bool rfS0bOff = switchArray[4]==1 || switchArray[4]==3 || switchArray[4]==5 || switchArray[4]==6;
		if (myReadBoolArray("Linac_mod_rf", stat)) {
			bool rfOn = false;
			bool rfOff = false;
			for (unsigned i=3; i<stat.size(); i++) {
				rfOn = rfOn || !stat[i];
			}
			for (unsigned i=0; i<4; i++) {
				rfOn = rfOn || !stat2[i];
			}
			rfOn = rfOn || rfPcGunOn || rfS0aOn || rfS0bOn;
			rfOff = rfPcGunOff || rfS0aOff || rfS0bOff;
			mySetTextColor(ui.linacRfTunnelStatLabel, (rfOn? "ON": (rfOff? "OFF": "???")), (rfOn? "#e00000": (rfOff? "#00b000": "black")));
		}
		else {
			mySetTextColor(ui.linacRfTunnelStatLabel, "???", "black");
		}
	}
	else {
		mySetTextColor(ui.linacRfPcgunStatLabel, "???", "black");
		mySetTextColor(ui.linacRfTunnelStatLabel, "???", "black");
	}
	/* set search buttons
	if (myReadBoolArray("Linac_search_lights", stat)) {
		ui.rb_lt01->setPixmap(QPixmap(imgpath+(stat[3]? "watch_button_on.png": "watch_button_white.png")));
		ui.rb_lt02->setPixmap(QPixmap(imgpath+(stat[0]? "watch_button_on.png": "watch_button_white.png")));
		ui.rb_lt03->setPixmap(QPixmap(imgpath+(stat[1]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_lt04->setPixmap(QPixmap(imgpath+(stat[4]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_lt05->setPixmap(QPixmap(imgpath+(stat[5]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_lt06->setPixmap(QPixmap(imgpath+(stat[6]? "watch_button_on.png": "watch_button_white.png")));
		ui.rb_lt07->setPixmap(QPixmap(imgpath+(stat[2]? "watch_button_on.png": "watch_button_white.png")));
		ui.rb_e0->setPixmap(QPixmap(imgpath+(stat[7]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_e1->setPixmap(QPixmap(imgpath+(stat[8]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_e2->setPixmap(QPixmap(imgpath+(stat[9]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_e3->setPixmap(QPixmap(imgpath+(stat[10]? "watch_button_on.png": "watch_button_white.png")));
		// ui.rb_tl->setPixmap(QPixmap(imgpath+(stat[11]? "watch_button_on.png": "watch_button_white.png")));
	}
	else {
		ui.rb_lt01->setPixmap(QPixmap(imgpath+""));
		ui.rb_lt02->setPixmap(QPixmap(imgpath+""));
		ui.rb_lt03->setPixmap(QPixmap(imgpath+""));
		// ui.rb_lt04->setPixmap(QPixmap(imgpath+""));
		// ui.rb_lt05->setPixmap(QPixmap(imgpath+""));
		// ui.rb_lt06->setPixmap(QPixmap(imgpath+""));
		ui.rb_lt07->setPixmap(QPixmap(imgpath+""));
		ui.rb_e0->setPixmap(QPixmap(imgpath+""));
		// ui.rb_e1->setPixmap(QPixmap(imgpath+""));
		// ui.rb_e2->setPixmap(QPixmap(imgpath+""));
		// ui.rb_e3->setPixmap(QPixmap(imgpath+""));
		// ui.rb_tl->setPixmap(QPixmap(imgpath+""));
	}
	*/
return;
	/*
	QSettings settings("fermi", "acs");
	// load control location
	if (settings.contains("controlLocation")) {
		controlLocation = settings.value("controlLocation").toString();
		// cout << "controlLocation: " << controlLocation.toStdString() << endl;
	}
	// update control location	
	// ui.controlLocationLabel->setText(controlLocation);
	*/
}

void acs::open_bst_linac()
{
	vector<bool> stat;
	// check if button is enabled
	if (myReadAttribute("BstB_13_1")) {
		myAttribute >> stat; 
		if (!stat[7]) {
			return;
		}
	}
	else {
		return;
	}
	// open procedure
	try {
		int r = QMessageBox::question(this, "Conferma apertura", "Conferma apertura Beam Stopper BST_B14.1", "Apri", "Annulla");
		if (r == 0) {
			try {
				myWriteCommand("Open_bst_b13_1");
			}
			catch (...) {
			}
		}
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Non posso aprire BST_B14.1" << endl;
	}
	cout << "open_bst" << endl;
}

void acs::close_bst_linac()
{
	// open procedure
	try {
		int r = QMessageBox::question(this, "Conferma chiusura", "Conferma chiusura Beam Stopper BST_B14.1", "Chiudi", "Annulla");
		if (r == 0) {
			try {
				myWriteCommand("Close_bst_b13_1");
			}
			catch (...) {
			}
		}
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Non posso chiudere BST_B14.1" << endl;
	}
	cout << "close_bst" << endl;
}

void acs::switch_master()
{
	// load (or set) master
	QSettings settings("fermi", "acs");
	if (settings.contains("masterlist")) {
		if (getenv("DISPLAY")==settings.value("masterlist").toString() && settings.contains("master")) {
			if (settings.value("master").toString().contains(getenv("DISPLAY"))) {
				master = false;
				settings.setValue("master", "none");
			}
			else {
				master = true;
				settings.setValue("master", getenv("DISPLAY"));
			}

		}		
		// cout << master.toStdString() << endl;
	}
	else {
		settings.setValue("masterlist", "qua:0");
	}
}


void acs::open_scrapersinfo_panel()
{
	if (scrapersinfo_process->state() != QProcess::Running) {
		scrapersinfo_process->kill();
		scrapersinfo_process->waitForFinished(3000);			
	}
	scrapersinfo_process->start("acsscrapersinfo"+style);	
}

void acs::open_alarmlist_panel()
{
	alarmlist_process->start("firefox 192.168.205.100/docs/hdb/acsadmin.php");	
}

void acs::switch_control_location()
{
	QSettings settings("fermi", "acs");
	// load control location
	controlLocation = settings.value("controlLocation").toString();
	if (controlLocation == "Sala Controllo Anello") {
		settings.setValue("controlLocation", "PSA");
	}
	else {
		settings.setValue("controlLocation", "Sala Controllo Anello");
	}
}

void acs::open_version_panel()
{
	vector<bool> stat;
	string strstat;
	// set PLC status
	if (myReadAttribute("Version")) {
		myAttribute >> strstat;
		strstat += "pan_ver: 1.30";
		// open procedure
		try {
			QMessageBox::question(this, "Versione", strstat.c_str(), "Esci");
		}
		catch (const Tango::DevFailed &e) {
			timestamp();
			cout << tmBuffer << "Non posso leggere la versione" << endl;
		}
	}
	else {
		timestamp();
		cout << tmBuffer << "Non posso leggere la versione" << endl;
	}		
}
